GlyphRecolor = LibStub("AceAddon-3.0"):NewAddon("GlyphRecolor", "AceConsole-3.0", "AceHook-3.0", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("GlyphRecolor", true)
local Gratuity = LibStub("LibGratuity-3.0")

local _MATCH_CLASSES = ""
local _REVERSE_CLASSES = {}

local db

local defaults = {
    profile = {
        showred = true,
		showicon = true,
		showborder = false,
    },
}

local _CLASS_ICONS = {
	["WARRIOR"] = {0, 0.25, 0, 0.25},
	["MAGE"] = {0.25, 0.49609375, 0, 0.25},
	["ROGUE"] = {0.49609375, 0.7421875, 0, 0.25},
	["DRUID"] = {0.7421875, 0.98828125, 0, 0.25},
	["HUNTER"] = {0, 0.25, 0.25, 0.5},
	["SHAMAN"] = {0.25, 0.49609375, 0.25, 0.5},
	["PRIEST"] = {0.49609375, 0.7421875, 0.25, 0.5},
	["WARLOCK"] = {0.7421875, 0.98828125, 0.25, 0.5},
	["PALADIN"] = {0, 0.25, 0.5, 0.75},
	["DEATHKNIGHT"] = {0.25, 0.49609375, 0.5, 0.75},
}

local function buildOptions()
	-- GUI options table
	local options = {
		type = "group",
		icon = "Interface\\Icons\\Trade_Engraving",
		name = "GlyphRecolor",
		get = function( k ) return db[k.arg] end,
		set = function( k, v ) db[k.arg] = v end,
		args = {
			description = {
				type = "description", order = 0,
				name = L["GlyphRecolor will show class icons on glyphs and color their borders. Also unusable glyphs are colored in red."],
			},
			showborder = {
				name = L["Color glyph border"],
				desc = L["Show glyph borders in class colors."],
				type = "toggle",
				arg = "showborder",
				order = 1, width = "full",
			},
			showicon = {
				name = L["Show class icons"],
				desc = L["Show class icons on glyphs."],
				type = "toggle",
				arg = "showicon",
				order = 2, width = "full",
			},
			showred = {
				name = L["Color unusable glyphs"],
				desc = L["Show unusable glyphs in red color."],
				type = "toggle",
				arg = "showred",
				order = 3, width = "full",
			},
		}
	}
	return options
end

function GlyphRecolor:OnInitialize()
	_MATCH_CLASSES = string.gsub(ITEM_CLASSES_ALLOWED, "%%s", "(.+)")
	
	_REVERSE_CLASSES = {}
	for k, v in pairs(RAID_CLASS_COLORS) do
		_REVERSE_CLASSES[(L[k])] = k
	end
	
	self:RegisterEvent("ADDON_LOADED")
	
	self:HookCombuctor()
	self:HookBlizzBags()
	
	self.db = LibStub("AceDB-3.0"):New("GlyphRecolorDB", defaults)
    db = self.db.profile
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("GlyphRecolor", buildOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GlyphRecolor", "GlyphRecolor")
	
    self:RegisterChatCommand("glyphrecolor", function() InterfaceOptionsFrame_OpenToCategory(LibStub("AceConfigDialog-3.0").BlizOptions["GlyphRecolor"].frame) end)
    self:RegisterChatCommand("glrec", function() InterfaceOptionsFrame_OpenToCategory(LibStub("AceConfigDialog-3.0").BlizOptions["GlyphRecolor"].frame) end)	
end

function GlyphRecolor:HookBlizzBags()
	self:SecureHook("ContainerFrame_Update")
	self:SecureHook("BankFrameItemButton_Update")
end

function GlyphRecolor:HookGuildBank()
	if GuildBankFrame_Update then self:SecureHook("GuildBankFrame_Update") end
end

function GlyphRecolor:HookCombuctor()
	if Combuctor and not self:IsHooked(CombuctorItem, "UpdateBorder") then
		self:SecureHook(CombuctorItem, "UpdateBorder")
	end
end

function GlyphRecolor:ADDON_LOADED(eventName, addonName)
	if (addonName == "Blizzard_GuildBankUI") then
		self:HookGuildBank()
	end
	
	if (addonName == "Combuctor") then
		self:HookCombuctor()
	end
end

function GlyphRecolor:GetGlyphData(link)
    if not string.find(link, "Glyph") then
		return false, false, ""
	end
	
	Gratuity:SetHyperlink(link)
    
	local Lines = Gratuity:GetText(2, 10, false, true)
    
	local class, glyphtype, file = false, false, ""
    
    if (Lines) then
        for num, line in pairs(Lines) do
            if (line[1] == MINOR_GLYPH) then
                glyphtype = "minor"
            elseif (line[1] == MAJOR_GLYPH) then
                glyphtype = "major"
            else
                class = string.match(line[1], _MATCH_CLASSES) or class
            end 
        end
    end
	
	if class then
		file = _REVERSE_CLASSES[class]
	end
	
    return class, glyphtype, file
end

function GlyphRecolor:PrepareButton(button, hideBorder, hideIcon)
	if hideBorder == nil then 
		hideBorder = false 
	end
	
	if hideIcon == nil then 
		hideIcon = true 
	end

	if hideBorder and button.border then
		button.border:Hide()
	end
	
	if hideIcon and button.classicon then
		button.classicon:Hide()
	end
	
	if InCombatLockdown() then return end
	
	if self.db.profile.showborder and not button.border then
		button.border = button:CreateTexture(nil, "OVERLAY")
		button.border:SetTexture('Interface/Buttons/UI-ActionButton-Border')
		button.border:SetWidth(67)
		button.border:SetHeight(67)
		button.border:SetPoint("CENTER", button)
		button.border:SetBlendMode('ADD')
		button.border:Hide()
	end    
	
	if self.db.profile.showicon and not button.classicon then
		button.classicon = button:CreateTexture(nil, "OVERLAY")
		button.classicon:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
		button.classicon:SetWidth(16)
		button.classicon:SetHeight(16)
		button.classicon:SetPoint("TOPLEFT", button, 3, -3)
		button.classicon:Hide()
	end
end

function GlyphRecolor:ColorButton(button, classFile)
	local color = RAID_CLASS_COLORS[classFile]
	
	if not color then return end
	
	if self.db.profile.showicon and button.classicon then
		local class = _CLASS_ICONS[classFile]
		button.classicon:SetTexCoord(class[1], class[2], class[3], class[4])
		button.classicon:Show()
	end
	
	if self.db.profile.showborder and button.border then
		button.border:SetVertexColor(color.r, color.g, color.b, 0.66)
	    button.border:Show()
	end
		
	if self.db.profile.showred and (classFile ~= select(2, UnitClass("player"))) then
		SetItemButtonTextureVertexColor(button, 1.0, 0.0, 0.0)
	end
end

-- Blizzard Bank
function GlyphRecolor:BankFrameItemButton_Update(button)
	if button.isBag then return end
	self:PrepareButton(button, true, true)
	
	if button.hasItem == 1 then
		local inventoryID = button:GetInventorySlot();
		local item = GetInventoryItemLink(inventoryID);
		
		if item then
			local class, glyphtype, file = self:GetGlyphData(item)
			if (class and glyphtype) then
				self:ColorButton(button, file)
			end
		end
	end
end

-- Blizzard Bags
function GlyphRecolor:ContainerFrame_Update(frame)
	local name = frame:GetName();
	local id = frame:GetID();
	
	for i=1, frame.size, 1 do
		itemButton = getglobal(name.."Item"..i);
		self:PrepareButton(itemButton, true, true)
		
		if itemButton.hasItem == 1 then 	
			item = GetContainerItemLink(id, itemButton:GetID())

			if item then
				local class, glyphtype, file = self:GetGlyphData(item)
				if (class and glyphtype) then
					self:ColorButton(itemButton, file)
				end
			end
		end
	end
end

-- Blizzard GuildBank
function GlyphRecolor:GuildBankFrame_Update()
	local tabnum = GetCurrentGuildBankTab()
    
    for column = 1, 7 do
        for slot = 1, 14 do
            local button = getglobal("GuildBankColumn" .. column .. "Button" .. slot)
            self:PrepareButton(button, true)
			
			local item = GetGuildBankItemLink(tabnum, button:GetID())
            
            if item then
                local class, glyphtype, file = self:GetGlyphData(item)
                if (class and glyphtype) then
                    self:ColorButton(button, file)
                end
            end
        end
    end
end

-- Combuctor
function GlyphRecolor:UpdateBorder(combuctor, quality)
	local link = combuctor.hasItem
	
	self:PrepareButton(combuctor)
	
	if link then
		local class, glyphtype, file = self:GetGlyphData(link)
		
		if (class and glyphtype) then
			self:ColorButton(combuctor, file)
		end
	end
end
